# pylint: disable=R0201,R0914

import os
import re
from importlib import import_module

from django.apps import apps
from django.conf import settings
from django.core.management.base import (
    BaseCommand,
    CommandError,
)

from baseapp.management.template_structures import (
    admins as admin_templates,
)
from baseapp.management.template_structures import (
    models as model_templates,
)

TEMPLATE_MODELS = {
    'django': model_templates.TEMPLATE_MODEL_DJANGO,
    'basemodel': model_templates.TEMPLATE_MODEL_BASEMODEL,
    'softdelete': model_templates.TEMPLATE_MODEL_SOFTDELETEMODEL,
}

TEMPLATE_ADMINS = {
    'django': admin_templates.TEMPLATE_ADMIN_DJANGO,
    'basemodel': admin_templates.TEMPLATE_ADMIN_BASEMODEL,
    'softdelete': admin_templates.TEMPLATE_ADMIN_SOFTDELETEMODEL,
}

USER_REMINDER = """

    `{model_name}` related files created successfully:

    - `{app_name}/models/{model_name_lower}.py`
    - `{app_name}/admin/{model_name_lower}.py`

    Please check your models before running `makemigrations` ok?

"""


class Command(BaseCommand):
    help = 'Creates models/MODEL.py, admin/MODEL.py for given application'  # noqa: A003

    MODEL_TYPE_CHOISES = ['django', 'basemodel', 'softdelete']

    def create_or_modify_file(self, filename, content, mode='w'):
        with open(filename, mode) as file_pointer:
            file_pointer.write(content)

    def add_arguments(self, parser):
        parser.add_argument('app_name', nargs=1, type=str, help='Name of your application')
        parser.add_argument('model_name', nargs=1, type=str, help='Name of your model')
        parser.add_argument(
            'model_type', nargs='?', default='django', choices=self.MODEL_TYPE_CHOISES, help='Type of your model'
        )

    def handle(self, *args, **options):
        app_name = options.pop('app_name')[0]

        model_name = options.pop('model_name')[0].lower()
        model_name_title = model_name.title()
        model_type = options.pop('model_type')

        try:
            import_module(app_name)
        except ImportError:
            raise CommandError('%s does not exists. Please pass existing application name.' % app_name)

        if model_name.lower() in [model.__name__.lower() for model in apps.get_app_config(app_name).get_models()]:
            raise CommandError(
                '%s model is already exists in %s. Please try non-existing model name.' % (model_name, app_name)
            )

        app_dir = os.path.join(settings.BASE_DIR, 'applications', app_name)

        dash_seperated_file_base_name = '_'.join([m for m in re.split('([A-Z][a-z]+)', model_name) if m])

        model_file = os.path.join(app_dir, 'models', '{0}.py'.format(dash_seperated_file_base_name.lower()))
        model_init_file = os.path.join(app_dir, 'models', '__init__.py')

        admin_file = os.path.join(app_dir, 'admin', '{0}.py'.format(dash_seperated_file_base_name.lower()))
        admin_init_file = os.path.join(app_dir, 'admin', '__init__.py')

        content_model_file = TEMPLATE_MODELS[model_type].format(
            model_name=model_name, app_name=app_name, model_name_title=model_name_title
        )
        content_init_file = 'from .{0} import *\n'.format(dash_seperated_file_base_name.lower())

        content_admin_file = TEMPLATE_ADMINS[model_type].format(
            model_name=model_name, app_name=app_name, model_name_title=model_name_title
        )

        self.create_or_modify_file(model_file, content_model_file)
        self.stdout.write(self.style.SUCCESS('models/{0} created.'.format(os.path.basename(model_file))))

        self.create_or_modify_file(admin_file, content_admin_file)
        self.stdout.write(self.style.SUCCESS('admin/{0} created.'.format(os.path.basename(admin_file))))

        self.create_or_modify_file(model_init_file, content_init_file, 'a')
        self.stdout.write(self.style.SUCCESS('{0} model added to models/__init__.py'.format(model_name)))

        self.create_or_modify_file(admin_init_file, content_init_file, 'a')
        self.stdout.write(self.style.SUCCESS('{0} model added to admin/__init__.py'.format(model_name)))

        self.stdout.write(
            self.style.NOTICE(
                USER_REMINDER.format(
                    app_name=app_name, model_name=model_name, model_name_lower=dash_seperated_file_base_name.lower()
                )
            )
        )
